@@ -41,6 +41,10 @@ gint gi_gem_clicked = 0;
 gint gi_x_click = 0;
 gint gi_y_click = 0;
 
+gint gi_gem_dragged = 0;
+gint gi_x_drag = 0;
+gint gi_y_drag = 0;
+
 gchar gpc_game_board[BOARD_WIDTH][BOARD_HEIGHT];
 gint gi_nb_of_tiles[7];
 
@@ -512,7 +516,7 @@ board_engine_loop (gpointer data)
 	static guint steps_for_timer = FIRST_BONUS_AT / TOTAL_STEPS_FOR_TIMER;
 
 	time_slice++;
-
+	
 // progressive score
 	if(gi_current_score < gi_score)
 	{
@@ -550,6 +554,7 @@ board_engine_loop (gpointer data)
 					gi_tile_height * y1,
 					2, gi_cursor_pixbuf);
 			gi_gem_clicked = 0;
+			gi_gem_dragged = 0;
 		}
 		break;
 
@@ -574,12 +579,18 @@ board_engine_loop (gpointer data)
 						y1 * gi_tile_height);
 				gi_state = _SECOND_GEM_CLICKED;
 			} else if((x1 == x2) && (y1 == y2)) {
+				if (cursor[1])
+					sge_destroy_object (cursor[1], NULL);
+				cursor[1] = NULL;					
 				// If the player clicks the selected gem, deselect it
 				if(cursor[0])
 					sge_destroy_object(cursor[0], NULL);
 				gi_state = _IDLE;
 				gi_gem_clicked = 0;
 			} else {
+				if (cursor[1])
+					sge_destroy_object (cursor[1], NULL);
+				cursor[1] = NULL;					
 				// If the player clicks anywhere else, make that the first selection
 				x1 = x2;
 				y1 = y2;
@@ -589,6 +600,14 @@ board_engine_loop (gpointer data)
 						gi_tile_height * y1,
 						2, gi_cursor_pixbuf);
 			}
+		}else if(gi_gem_dragged)
+		{
+			//printf("gem dragged\n");
+			if (cursor[1])
+					sge_destroy_object (cursor[1], NULL);
+				cursor[1] = sge_create_object (gi_tile_width * gi_x_drag,
+						gi_tile_height * gi_y_drag,
+						2, gi_cursor_pixbuf);
 		}
 		break;
 
